class InstallEribium < ActiveRecord::Migration
  def self.up
  puts "Installing Eribium"
   create_table "cached_feeds", :force => true do |t|
      t.column "href", :string
      t.column "title", :string
      t.column "link", :string
      t.column "feed_data", :text
      t.column "feed_data_type", :string, :limit => 20
      t.column "http_headers", :text
      t.column "last_retrieved", :datetime
    end
  
    create_table "flares", :force => true do |t|
      t.column "title", :string, :default => "", :null => false
      t.column "description", :text, :default => "", :null => false
      t.column "url", :string, :default => "", :null => false
      t.column "created_at", :datetime, :null => false
      t.column "updated_at", :datetime, :null => false
    end
    
    Flare.create(:title => "Test Flare", :url => "Test Flare")
  
    create_table "pages", :force => true do |t|
      t.column "name", :string, :limit => 45, :default => "", :null => false
      t.column "url", :string, :limit => 45, :default => "", :null => false
      t.column "meta_tags", :text, :default => "", :null => false
      t.column "meta_descs", :text, :default => "", :null => false
      t.column "created_at", :datetime, :null => false
      t.column "updated_at", :datetime, :null => false
      t.column "body", :text, :default => "", :null => false
      t.column "behaviour", :text, :default => "", :null => false
      t.column "body_html", :text, :default => "", :null => false
      t.column "created_by", :string, :default => "", :null => false
      t.column "updated_by", :string, :default => "", :null => false
      t.column "text_filter", :string, :default => "", :null => false
      t.column "show_in_menu", :integer, :limit => 10, :default => 0, :null => false
    end
    
    Page.create(:name => "Welcome to Eribium", :created_by => "Alex MacCaw", :url => "welcome_to_eribium", :body => "Default username and password are admin, you can login <a href='/admin'>here</a>.", :text_filter => 'markdown', :show_in_menu => 1)
  
    create_table "pages_tags", :id => false, :force => true do |t|
      t.column "page_id", :integer, :limit => 10, :default => 0, :null => false
      t.column "tag_id", :integer, :limit => 10, :default => 0, :null => false
    end
  
    create_table "preferences", :force => true do |t|
      t.column "setting", :string, :default => "", :null => false
      t.column "value", :string, :default => "", :null => false
    end
    
    puts "Now Installing Settings"
    
       Preference.create(:setting => "site_title", :value => "Eribium")
       Preference.create(:setting => "site_subtitle", :value => "A Rails CMS")
       Preference.create(:setting => "site_lat_long")
       Preference.create(:setting => "site_default_page")
       Preference.create(:setting => "site_default_text_filter", :value => "markdown")
       Preference.create(:setting => "theme", :value => "laughingatyou")
       Preference.create(:setting => "large_width")
       
       
       Preference.create(:setting => "page_order_by", :value => "name")
       Preference.create(:setting => "page_display_number", :value => 5)
       Preference.create(:setting => "page_display_order", :value => "desc")
       
       
       Preference.create(:setting => "feed_title")
       Preference.create(:setting => "feed_description")
       Preference.create(:setting => "feed_link")
       Preference.create(:setting => "feed_author")
       
       Preference.create(:setting => "feed_order_by", :value => "title")
       Preference.create(:setting => "feed_display_number", :value => 5)
       Preference.create(:setting => "feed_display_order", :value => "desc")
       
       Preference.create(:setting => "file_order_by", :value => "name")
       Preference.create(:setting => "file_display_number", :value => 5)
       Preference.create(:setting => "file_display_order", :value => "desc")
       
       Preference.create(:setting => "tag_order_by", :value => "name")
       Preference.create(:setting => "tag_display_number", :value => 5)
       Preference.create(:setting => "tag_display_order", :value => "desc")
       
       Preference.create(:setting => "stat_order_by", :value => "domain")
       Preference.create(:setting => "stat_display_number", :value => 5)
       Preference.create(:setting => "stat_display_order", :value => "desc")
  
    create_table "stats", :force => true do |t|
      t.column "remote_address", :string
      t.column "http_accept_language", :string
      t.column "http_user_agent", :string
      t.column "path_info", :string
      t.column "page_title", :string
      t.column "flash_version", :string
      t.column "resolution_x", :string
      t.column "resolution_y", :string
      t.column "viewport_x", :string
      t.column "viewport_y", :string
      t.column "http_referer", :string
      t.column "query_string", :string
      t.column "domain", :string
      t.column "created_at", :datetime, :null => false
      t.column "browser", :string, :default => "", :null => false
      t.column "browser_version", :string, :default => "", :null => false
    end
  
    create_table "tags", :force => true do |t|
      t.column "name", :string, :default => "", :null => false
      t.column "created_at", :datetime, :null => false
      t.column "updated_at", :datetime, :null => false
      t.column "description", :text, :default => "", :null => false
      t.column "description_html", :text, :default => "", :null => false
      t.column "text_filter", :string, :default => "", :null => false
    end
  
    create_table "uploads", :force => true do |t|
      t.column "description", :text, :default => "", :null => false
      t.column "created_at", :datetime, :null => false
      t.column "updated_at", :datetime, :null => false
      t.column "name", :string, :limit => 200, :default => "", :null => false
      t.column "file", :text, :default => "", :null => false
    end
  
    create_table "users", :force => true do |t|
      t.column "name", :string, :limit => 100, :default => "", :null => false
      t.column "hashed_password", :string, :limit => 100, :default => "", :null => false
      t.column "created_at", :datetime, :null => false
      t.column "updated_at", :datetime, :null => false
      t.column "email", :string, :limit => 45, :default => "", :null => false
      t.column "first_name", :string, :limit => 45, :default => "", :null => false
      t.column "last_name", :string, :limit => 45, :default => "", :null => false
      t.column "description", :text, :default => "", :null => false
      t.column "photo", :string, :limit => 45, :default => "", :null => false
    end
    User.create(:name => "admin", :password => "admin", :password_confirmation => "admin")
    puts "Created user, username: admin, password: admin"
    
    puts "Installed Eribium successfully, browse to http://localhost:3000/"
  end

  def self.down
  
      drop_table :cached_feeds
      drop_table :flares
      drop_table :pages
      drop_table :pages_tags
      drop_table :preferences
      drop_table :stats
      drop_table :tags
      drop_table :uploads
      drop_table  users
    
  end
end
